/*********************************************************************************

   Author: Artur Szostak
   Email:  artur@alice.phy.uct.ac.za

   Declarations of for the irqsignal kernel module.

**********************************************************************************/

#ifndef IRQ_SIGNAL_KERNEL_MODULE_H
#define IRQ_SIGNAL_KERNEL_MODULE_H

// The maximum valid IRQ number that our module accepts.
#define MAX_NUMBER_OF_IRQS NR_IRQS


// The possible error codes returned by this module on any of the ioctl calls.
typedef enum irqsignal_status
{
	IRQSIG_OK = 0,
	IRQSIG_UNKNOWN_ERROR = -1,
	IRQSIG_NO_SUCH_IOCTRL = 100,
	IRQSIG_OUTOFMEM = 101,
	IRQSIG_BADIRQ = 200,
	IRQSIG_ALREADY_REGISTERED = 201,
	IRQSIG_IRQ_IN_USE = 202,
	IRQSIG_NOT_REGISTERED = 300,
	IRQSIG_CANT_SCHEDULE_TASK = 400,

	IRQSIG_CANT_OPEN_DRIVER = 20  // This error code is actually returned by the API.
} irqsignal_status_t;


typedef struct irqsignal_register_info
{
	unsigned int irq; // The interrupt line the process is interested in.
	int signal;       // the signal to send to the process.
} irqsignal_register_info_t;


#include <asm/ioctl.h>

// The base IOCTL number.
#define IRQSIGNAL_TYPE_NUMBER 'I'

// All the actuall IOCTL numbers used by the kernel module.
#define IRQSIGNAL_REGISTER			_IOC( _IOC_READ, IRQSIGNAL_TYPE_NUMBER, 0, sizeof(irqsignal_register_info_t) )
#define IRQSIGNAL_UNREGISTER		_IOC( _IOC_READ, IRQSIGNAL_TYPE_NUMBER, 1, sizeof(irqsignal_register_info_t) )
#define IRQSIGNAL_UNREGISTER_ALL	_IOC( _IOC_READ, IRQSIGNAL_TYPE_NUMBER, 2, sizeof(unsigned int) )
#define IRQSIGNAL_RESETIRQ			_IOC( _IOC_NONE, IRQSIGNAL_TYPE_NUMBER, 3, sizeof(unsigned int) )
#define IRQSIGNAL_RESETIRQALL		_IOC( _IOC_NONE, IRQSIGNAL_TYPE_NUMBER, 4, 0 )
#define IRQSIGNAL_HARDRESET			_IOC( _IOC_NONE, IRQSIGNAL_TYPE_NUMBER, 5, 0 )
#define IRQSIGNAL_TEST				_IOC( _IOC_READ, IRQSIGNAL_TYPE_NUMBER, 6, sizeof(unsigned int) )


#endif // IRQ_SIGNAL_KERNEL_MODULE_H
