/*******************************************************************************

  Author: Artur Szostak
  Email:  artur@alice.phy.uct.ac.za  |  artursz@iafrica.com
  
  Implementation of FetchIRQLineByVendor and FetchIRQLineByBus routines.

********************************************************************************/

#include "fetchIRQline.h"
#include "psi_error.h"

PSI_Status FetchIRQLineByVendor(
		const unsigned short vendID, const unsigned short devID, const unsigned int card,
		unsigned char* irqline
	)
{
	tRegion region;
	PSI_Status psiStatus;
	char name[128];

	// Open PCI configuration region by vendor / device ID / card number
	sprintf(name, "/dev/psi/vendor/0x%04hX/device/0x%04hX/0x%08lX/conf", vendID, devID, card);
	psiStatus = PSI_openRegion( &region, name );
	if ( psiStatus != PSI_OK ) return psiStatus;
	
	// Read the IRQ line byte #60.
	psiStatus = PSI_read(region, 60, 1, 1, irqline);
	
	PSI_closeRegion( region );
	return psiStatus;
};


PSI_Status FetchIRQLineByBus(
		const unsigned short bus, const unsigned short dev, const unsigned short func,
		unsigned char* irqline
	)
{
	tRegion region;
	PSI_Status psiStatus;
	char name[128];

	// Open PCI configuration region by bus / device / function
	sprintf(name, "/dev/psi/bus/0x%04hX/slot/0x%04hX/function/0x%04hX/conf", bus, dev, func);
	psiStatus = PSI_openRegion( &region, name );
	if ( psiStatus != PSI_OK ) return psiStatus;

	// Read the IRQ line byte #60.
	psiStatus = PSI_read(region, 60, 1, 1, irqline);

	PSI_closeRegion( region );
	return psiStatus;
};

