--=============================================================================
-- Project    : HLT-RORC
-------------------------------------------------------------------------------
-- File       : event_monitor.vhd
-- Author     : Heinz Tilsner  <tilsner@kip.uni-heidelberg.de>
-- Company    : 
-- Platform   : 
-------------------------------------------------------------------------------
-- Description:
--
--
-------------------------------------------------------------------------------
-- Revision list:
-- Date        Version  Author    Description
--
-- 03.11.2004  1.0      tilsner   created
--=============================================================================

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_signed.ALL;

ENTITY event_monitor IS
  GENERIC (
    evmon_counter : integer RANGE 0 TO 255 := 0
  );
  PORT (
    clk          : IN  std_logic;
    rstn         : IN  std_logic;
    data         : IN  std_logic_vector(64 DOWNTO 0);
    eoe_count    : OUT std_logic_vector(7 DOWNTO 0);
    event_count  : OUT std_logic_vector(15 DOWNTO 0);

    -- config IF
    conf_addr    : IN  std_logic_vector(7 DOWNTO 0);
    conf_in      : IN  std_logic_vector(31 DOWNTO 0);
    conf_out     : OUT std_logic_vector(31 DOWNTO 0);
    conf_we      : IN  std_logic
  );

END event_monitor;

ARCHITECTURE rtl OF event_monitor IS

  SIGNAL eoe_flag      : std_logic;
  SIGNAL eoe_counter   : std_logic_vector(7 DOWNTO 0);
  SIGNAL event_counter : std_logic_vector(15 DOWNTO 0);

BEGIN

  eoe_flag    <= data(64);
  eoe_count   <= eoe_counter;
  event_count <= event_counter;

  ---------------------------------------------------------------------------
  -- configuration interface:
  ---------------------------------------------------------------------------
  conf_out <= (x"00" & eoe_counter & event_counter) WHEN (conf_addr = evmon_counter) ELSE (OTHERS => '0');

  
  -----------------------------------------------------------------------------
  -- counter for eoe flags
  -----------------------------------------------------------------------------
  PROCESS (clk, rstn)
    BEGIN
      IF (rstn='0') THEN
        eoe_counter <= (OTHERS => '0');
      ELSIF rising_edge(clk) THEN
        IF (eoe_flag = '1') THEN
          eoe_counter <= eoe_counter + 1;
        ELSIF (data(64) = '1') THEN
          eoe_counter <= eoe_counter - 1;
        END IF;
      END IF;
    END PROCESS;
    
  -------------------------------------------------------------------------------
  -- event counter
  -------------------------------------------------------------------------------    
  PROCESS (clk, rstn)
  BEGIN
    IF (rstn = '0') THEN
      event_counter <= (OTHERS => '0');
    ELSIF (clk'event and clk = '1') THEN
      IF (eoe_flag = '1') THEN
        event_counter <= event_counter + '1';
      ELSE
        event_counter <= event_counter;
      END IF;
    END IF;
  END PROCESS;


END rtl;
