--=============================================================================
-- Project    : HLT-RORC
-------------------------------------------------------------------------------
-- File       : packetizer.vhd
-- Author     : Heinz Tilsner  <tilsner@kip.uni-heidelberg.de> / Olav Torheim <torheim@ift.uib.no>
-- Company    :
-- Platform   :
-------------------------------------------------------------------------------
-- Description:
--
--
-------------------------------------------------------------------------------
-- Revision list:
-- Date        Version  Author      Description
--
-- 07.07.2004  1.0      tilsner     created
--
-- 24.06.2005  1.1      O. Torheim  Did some minor changes required for handling abnormal transaction terminations on the PCI bus
--=============================================================================

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_unsigned.ALL;

ENTITY packetizer IS

  PORT (
    clk        : IN std_logic;
    rstn       : IN std_logic;
    rdreq_in   : IN std_logic;
    rdreq_out  : OUT std_logic;
    burst_continue   : IN std_logic;        
    mstr_done  : IN std_logic;
    last_block : OUT std_logic;
    data_in    : IN std_logic_vector(64 DOWNTO 0);
    data_out   : OUT std_logic_vector(63 DOWNTO 0)
  );

END packetizer;


ARCHITECTURE syn OF packetizer IS

  TYPE state_type IS (normal, trans_status, zero);
  SIGNAL state : state_type;
  SIGNAL status_bit : std_logic;
  
  BEGIN
    last_block <= '1' WHEN (state=zero) ELSE '0';

    status_bit <= data_in(64);
    
--    data_out  <= (OTHERS => '0') WHEN (status_bit='1' OR state=zero) ELSE data_in(63 DOWNTO 0);
    data_out  <= (OTHERS => '0') WHEN (state=zero) ELSE data_in(63 DOWNTO 0);
    rdreq_out <= mstr_done WHEN (state=zero) ELSE
                 '0'       WHEN (status_bit='1') ELSE
                 rdreq_in WHEN burst_continue='0' ELSE '0';
    

--    PROCESS (clk, rstn)
--      BEGIN
--        IF (rstn='0') THEN
--          state <= normal;
--        ELSIF (clk'event AND clk='1') THEN
--          CASE state IS
--
--            WHEN normal => 
--              IF (status_bit='1') THEN 
--                state <= zero;
--              ELSE
--                state <= normal;
--              END IF;
--
--            WHEN zero => 
--              IF (mstr_done='1') THEN
--                state <= normal;
--              ELSE
--                state <= zero;
--              END IF;
--              
--            END CASE;
--        END IF;
--      END PROCESS;


    PROCESS (clk, rstn)
      BEGIN
        IF (rstn='0') THEN
          state <= normal;
        ELSIF (clk'event AND clk='1') THEN
          CASE state IS

            WHEN normal => 
              IF (status_bit='1' AND rdreq_in='0') THEN 
                state <= trans_status;
              ELSIF (status_bit='1' AND rdreq_in='1' AND burst_continue='0') THEN 
                state <= zero;
              ELSE
                state <= normal;
              END IF;

            WHEN trans_status =>
              IF (rdreq_in='1' AND burst_continue='0') THEN
                state <= zero;
              ELSE
                state <= trans_status;
              END IF;
              
            WHEN zero =>
              IF (mstr_done='1') THEN
                state <= normal;
              ELSE
                state <= zero;
              END IF;
              
            END CASE;
        END IF;
      END PROCESS;

END syn;
