--=============================================================================
-- Project    : HLT-RORC
-------------------------------------------------------------------------------
-- File       : pcim_top_r.vhd
-- Author     : Olav Torheim <torheim@ift.uib.no>
-- Company    :
-- Platform   :
-------------------------------------------------------------------------------
-- Description: This file was originally a VHDL template for instantiation of LogiCORE PCI applications, now fitted for serving as top module for HLT-RORC.
-- 
--
-------------------------------------------------------------------------------
-- Revision list:
-- Date        Version  Author      Description
-- 24.06.2005  1.0      O. Torheim  Replacing the original Reference Design with HLT-RORC.
--                                  Most of the added modules are cut and paste from the Altera-based top_module.vhd created by Heinz Tilsner.
=============================================================================


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

library UNISIM;
use UNISIM.VCOMPONENTS.ALL;

entity pcim_top is
  port (-- PCI ports; do not modify names!
        AD              : inout std_logic_vector(63 downto 0);
        CBE             : inout std_logic_vector( 7 downto 0);
        PAR             : inout std_logic;
        PAR64           : inout std_logic;
        FRAME_N         : inout std_logic;
        REQ64_N         : inout std_logic;
        TRDY_N          : inout std_logic;
        IRDY_N          : inout std_logic;
        STOP_N          : inout std_logic;
        DEVSEL_N        : inout std_logic;
        ACK64_N         : inout std_logic;
        IDSEL           : in    std_logic;
        INTR_A          : out   std_logic;
        PERR_N          : inout std_logic;
        SERR_N          : inout std_logic;
        REQ_N           : out   std_logic;
        GNT_N           : in    std_logic;
        RST_N           : in    std_logic;
        PCLK            : in    std_logic;
        OSCCLK            : in    std_logic;

        CLK_GLOBAL      : in    std_logic;

        -- Add user I/O ports here
	-- DDL interface
	roCLK       : OUT std_logic;
    	roRST_N     : OUT std_logic;
    	riLD_N      : IN  std_logic;
    	riLF_N      : IN  std_logic;
    	roBSY_N     : OUT std_logic;
    	roD         : OUT std_logic_vector(31 DOWNTO 0);
    	roTEN_N     : OUT std_logic;
    	roCMD_N     : OUT std_logic;
    	riD         : IN  std_logic_vector(31 DOWNTO 0);
    	riTEN_N     : IN  std_logic;
    	riSTS_N     : IN  std_logic;

    	-- 32-bit memory interaface
	ucc_d       : INOUT std_logic_vector(31 DOWNTO 0);
    	ucc_a       : OUT   std_logic_vector(26 DOWNTO 0);
    	ucc_a1_a21  : OUT   std_logic;
    	ucc_sram_cs : OUT   std_logic_vector(3 DOWNTO 0);
    	ucc_we      : OUT   std_logic;
    	ucc_be      : OUT   std_logic_vector(3 DOWNTO 0);
    	sram32_oe   : OUT   std_logic;

    	-- 64-bit memory interface
    	mem_a       : OUT   std_logic_vector(21 downto 0);
    	mem_d       : INOUT std_logic_vector(63 downto 0);
    	sram64_cs   : OUT   std_logic;
    	sram64_we   : OUT   std_logic_vector(1 DOWNTO 0);
    	sram64_be   : OUT   std_logic_vector(7 DOWNTO 0);
    	sram64_oe	: OUT   std_logic_vector(1 downto 0);
    	sram64_adsc	: OUT   std_logic_vector(1 downto 0)
  );
end pcim_top;


architecture rtl of pcim_top is

  attribute syn_edif_bit_format : string;
  attribute syn_edif_scalar_format : string;
  attribute syn_noclockbuf : boolean;
  attribute syn_hier : string;
  attribute syn_edif_bit_format of rtl : architecture is "%u<%i>";
  attribute syn_edif_scalar_format of rtl : architecture is "%u";
  attribute syn_noclockbuf of rtl : architecture is true;
  attribute syn_hier of rtl : architecture is "hard";

  -- Component declaration of PCI Interface
  component pcim_lc
  port (-- PCI ports; do not modify names!
        AD_IO           : inout std_logic_vector( 63 downto 0);
        CBE_IO          : inout std_logic_vector(  7 downto 0);
        PAR_IO          : inout std_logic;
        PAR64_IO        : inout std_logic;
        FRAME_IO        : inout std_logic;
        REQ64_IO        : inout std_logic;
        TRDY_IO         : inout std_logic;
        IRDY_IO         : inout std_logic;
        STOP_IO         : inout std_logic;
        DEVSEL_IO       : inout std_logic;
        ACK64_IO        : inout std_logic;
        IDSEL_I         : in    std_logic;
        INTA_O          : out   std_logic;
        PERR_IO         : inout std_logic;
        SERR_IO         : inout std_logic;
        REQ_O           : out   std_logic;
        GNT_I           : in    std_logic;
        RST_I           : in    std_logic;
        PCLK            : in    std_logic;
        RCLK            : in    std_logic;
        CFG             : in    std_logic_vector(255 downto 0);
        FRAMEQ_N        : out   std_logic;
        REQ64Q_N        : out   std_logic;
        TRDYQ_N         : out   std_logic;
        IRDYQ_N         : out   std_logic;
        STOPQ_N         : out   std_logic;
        DEVSELQ_N       : out   std_logic;
        ACK64Q_N        : out   std_logic;
        ADDR            : out   std_logic_vector( 31 downto 0);
        ADIO            : inout std_logic_vector( 63 downto 0);
        CFG_VLD         : out   std_logic;
        CFG_HIT         : out   std_logic;
        C_TERM          : in    std_logic;
        C_READY         : in    std_logic;
        ADDR_VLD        : out   std_logic;
        BASE_HIT        : out   std_logic_vector(  7 downto 0);
        S_CYCLE64       : out   std_logic;
        S_TERM          : in    std_logic;
        S_READY         : in    std_logic;
        S_ABORT         : in    std_logic;
        S_WRDN          : out   std_logic;
        S_SRC_EN        : out   std_logic;
        S_DATA_VLD      : out   std_logic;
        S_CBE           : out   std_logic_vector(  7 downto 0);
        PCI_CMD         : out   std_logic_vector( 15 downto 0);
        REQUEST         : in    std_logic;
        REQUEST64       : in    std_logic;
        REQUESTHOLD     : in    std_logic;
        COMPLETE        : in    std_logic;
        M_WRDN          : in    std_logic;
        M_READY         : in    std_logic;
        M_SRC_EN        : out   std_logic;
        M_DATA_VLD      : out   std_logic;
        M_CBE           : in    std_logic_vector(  7 downto 0);
        TIME_OUT        : out   std_logic;
        M_FAIL64        : out   std_logic;
        CFG_SELF        : in    std_logic;
        M_DATA          : out   std_logic;
        DR_BUS          : out   std_logic;
        I_IDLE          : out   std_logic;
        M_ADDR_N        : out   std_logic;
        IDLE            : out   std_logic;
        B_BUSY          : out   std_logic;
        S_DATA          : out   std_logic;
        BACKOFF         : out   std_logic;
        SLOT64          : in    std_logic;
        INTR_N          : in    std_logic;
        PERRQ_N         : out   std_logic;
        SERRQ_N         : out   std_logic;
        KEEPOUT         : in    std_logic;
        CSR             : out   std_logic_vector( 39 downto 0);
        SUB_DATA        : in    std_logic_vector( 31 downto 0);
        RST             : inout std_logic;
        CLK             : inout std_logic
  );
  end component;


  -- Component declaration for Configuration
  component CFG
  port (
        CFG             : out   std_logic_vector(255 downto 0)
  );
  end component;


  -------------------------------------------------------------------------------
  -- local side
  -------------------------------------------------------------------------------
  COMPONENT local_side
    PORT (
    --pci signals
        FRAMEQ_N        : in    std_logic;
        REQ64Q_N        : in    std_logic;
        TRDYQ_N         : in    std_logic;
        IRDYQ_N         : in    std_logic;
        STOPQ_N         : in    std_logic;
        DEVSELQ_N       : in    std_logic;
        ACK64Q_N        : in    std_logic;
        ADDR            : in    std_logic_vector( 31 downto 0);
        ADIO            : inout std_logic_vector( 63 downto 0);
        CFG_VLD         : in    std_logic;
        CFG_HIT         : in    std_logic;
        C_TERM          : out   std_logic;
        C_READY         : out   std_logic;
        ADDR_VLD        : in    std_logic;
        BASE_HIT        : in    std_logic_vector(  7 downto 0);
        S_CYCLE64       : in    std_logic;
        S_TERM          : out   std_logic;
        S_READY         : out   std_logic;
        S_ABORT         : out   std_logic;
        S_WRDN          : in    std_logic;
        S_SRC_EN        : in    std_logic;
        S_DATA_VLD      : in    std_logic;
        S_CBE           : in    std_logic_vector(  7 downto 0);
        PCI_CMD         : in    std_logic_vector( 15 downto 0);
        REQUEST         : out   std_logic;
        REQUEST64       : out   std_logic;
        REQUESTHOLD     : out   std_logic;
        COMPLETE        : out   std_logic;
        M_WRDN          : out   std_logic;
        M_READY         : out   std_logic;
        M_SRC_EN        : in    std_logic;
        M_DATA_VLD      : in    std_logic;
        M_CBE           : out   std_logic_vector(  7 downto 0);
        TIME_OUT        : in    std_logic;
        M_FAIL64        : in    std_logic;
        CFG_SELF        : out   std_logic;
        M_DATA          : in    std_logic;
        DR_BUS          : in    std_logic;
        I_IDLE          : in    std_logic;
        M_ADDR_N        : in    std_logic;
        IDLE            : in    std_logic;
        B_BUSY          : in    std_logic;
        S_DATA          : in    std_logic;
        BACKOFF         : in    std_logic;
        SLOT64          : out   std_logic;
        INTR_N          : out   std_logic;
        PERRQ_N         : in    std_logic;
        SERRQ_N         : in    std_logic;
        KEEPOUT         : out   std_logic;
        CSR             : in    std_logic_vector( 39 downto 0);
        SUB_DATA        : out   std_logic_vector( 31 downto 0);
        CFG             : in    std_logic_vector(255 downto 0);
        RST             : in    std_logic;
        CLK             : in    std_logic;
        CLK_REG         : in    std_logic;        

     -- DDL interface
    roCLK           : OUT std_logic;
    roRST_N         : OUT std_logic;
    riLD_N          : IN  std_logic;
    riLF_N          : IN  std_logic;
    roBSY_N         : OUT std_logic;
    roD             : OUT std_logic_vector(31 DOWNTO 0);
    roTEN_N         : OUT std_logic;
    roCMD_N         : OUT std_logic;
    riD             : IN  std_logic_vector(31 DOWNTO 0);
    riTEN_N         : IN  std_logic;
    riSTS_N         : IN  std_logic;

    -- 32-bit memory interaface
    sram32_a       : OUT   std_logic_vector(23 DOWNTO 0);
    sram32_d       : INOUT std_logic_vector(31 DOWNTO 0);
    sram32_we      : OUT   std_logic;
    sram32_be      : OUT   std_logic_vector(3 DOWNTO 0);
    sram32_oe      : OUT   std_logic;

    -- 64-bit memory interface
    sram64_a       : OUT   std_logic_vector(21 DOWNTO 0);
    sram64_d       : INOUT std_logic_vector(63 DOWNTO 0);
    sram64_we      : OUT   std_logic_vector(1 DOWNTO 0);
    sram64_be      : OUT   std_logic_vector(7 DOWNTO 0);
    sram64_oe      : OUT   std_logic_vector(1 DOWNTO 0);
    sram64_cs      : OUT   std_logic;

    -- status
    hostbuf_full    : OUT std_logic;
    dmabuf_full     : OUT std_logic;
    dma_enabled     : OUT std_logic;
    dma_active      : OUT std_logic;
    link_down       : OUT std_logic
    );
  END COMPONENT;

  component rclk_gen
   port ( CLKIN_IN        : in    std_logic;
          RST_IN          : in    std_logic;
          CLKFX_OUT       : out   std_logic;
          CLKIN_IBUFG_OUT : out   std_logic;
          CLK0_OUT        : out   std_logic;
          LOCKED_OUT      : out   std_logic);
   end component;

	COMPONENT global_phase_shift
	PORT(
		CLKIN_IN : IN std_logic;
		RST_IN : IN std_logic;          
		CLKIN_IBUFG_OUT : OUT std_logic;
		CLK0_OUT : OUT std_logic;
		CLK180_OUT : OUT std_logic;
		LOCKED_OUT : OUT std_logic
		);
	END COMPONENT;

     -----------------------------------------------------------------------------
  -- specify type of RAM on the RORC
  -----------------------------------------------------------------------------
  constant mem_type: string := "cypress";

  -- Internal signals; do not modify names!

  signal FRAMEQ_N       : std_logic;
  signal REQ64Q_N       : std_logic;
  signal TRDYQ_N        : std_logic;
  signal IRDYQ_N        : std_logic;
  signal STOPQ_N        : std_logic;
  signal DEVSELQ_N      : std_logic;
  signal ACK64Q_N       : std_logic;
  signal ADDR           : std_logic_vector( 31 downto 0);
  signal ADIO           : std_logic_vector( 63 downto 0);
  signal CFG_VLD        : std_logic;
  signal CFG_HIT        : std_logic;
  signal C_TERM         : std_logic;
  signal C_READY        : std_logic;
  signal ADDR_VLD       : std_logic;
  signal BASE_HIT       : std_logic_vector(  7 downto 0);
  signal S_CYCLE64      : std_logic;
  signal S_TERM         : std_logic;
  signal S_READY        : std_logic;
  signal S_ABORT        : std_logic;
  signal S_WRDN         : std_logic;
  signal S_SRC_EN       : std_logic;
  signal S_DATA_VLD     : std_logic;
  signal S_CBE          : std_logic_vector(  7 downto 0);
  signal PCI_CMD        : std_logic_vector( 15 downto 0);
  signal REQUEST        : std_logic;
  signal REQUEST64      : std_logic;
  signal REQUESTHOLD    : std_logic;
  signal COMPLETE       : std_logic;
  signal M_WRDN         : std_logic;
  signal M_READY        : std_logic;
  signal M_SRC_EN       : std_logic;
  signal M_DATA_VLD     : std_logic;
  signal M_CBE          : std_logic_vector(  7 downto 0);
  signal TIME_OUT       : std_logic;
  signal M_FAIL64       : std_logic;
  signal CFG_SELF       : std_logic;
  signal M_DATA         : std_logic;
  signal DR_BUS         : std_logic;
  signal I_IDLE         : std_logic;
  signal M_ADDR_N       : std_logic;
  signal IDLE           : std_logic;
  signal B_BUSY         : std_logic;
  signal S_DATA         : std_logic;
  signal BACKOFF        : std_logic;
  signal SLOT64         : std_logic;
  signal INTR_N         : std_logic;
  signal PERRQ_N        : std_logic;
  signal SERRQ_N        : std_logic;
  signal KEEPOUT        : std_logic;
  signal CSR            : std_logic_vector( 39 downto 0);
  signal SUB_DATA       : std_logic_vector( 31 downto 0);
  signal CFG_BUS        : std_logic_vector(255 downto 0);
  signal RST            : std_logic;
  signal CLK            : std_logic;

  SIGNAL CLKIN_IBUFG_OUT1 : STD_LOGIC;
  SIGNAL CLK0_OUT1        : STD_LOGIC;
  SIGNAL LOCKED_OUT1      : STD_LOGIC;

  SIGNAL CLKIN_IBUFG_OUT2 : STD_LOGIC;
  SIGNAL CLK0_OUT2        : STD_LOGIC;
  SIGNAL LOCKED_OUT2      : STD_LOGIC;

  SIGNAL RCLK      : STD_LOGIC;
  
  SIGNAL CLK_GLOBAL_180 : STD_LOGIC;

  SIGNAL sram32_a_int : std_logic_vector(23 DOWNTO 0);
  SIGNAL mem_a_int    : std_logic_vector(21 DOWNTO 0);
  SIGNAL leds         : STD_LOGIC_VECTOR(3 DOWNTO 0);
  SIGNAL hostbuf_full : STD_LOGIC;
  SIGNAL dmabuf_full  : STD_LOGIC;
  SIGNAL dma_enabled  : STD_LOGIC;
  SIGNAL dma_active   : STD_LOGIC;
  SIGNAL link_down    : STD_LOGIC;
  

begin

  -- SRAM 32bit interface
  ucc_a(26 DOWNTO 21) <= (OTHERS => '0');
  ucc_a(20 DOWNTO 0)  <= sram32_a_int(20 DOWNTO 0);

  addr_micron: IF (mem_type = "micron") GENERATE
    BEGIN
      ucc_a1_a21  <= sram32_a_int(20);
      ucc_sram_cs <= ("11" & (NOT sram32_a_int(21)) & sram32_a_int(21));
    END GENERATE;

  addr_cypress: IF (mem_type = "cypress") GENERATE
    BEGIN
      ucc_a1_a21  <= sram32_a_int(21);
      ucc_sram_cs <= ("11" & (NOT sram32_a_int(22)) & sram32_a_int(22));
    END GENERATE;

  -- REFCLK Generation:

    RCLK_GEN0 : rclk_gen
   port map ( CLKIN_IN        =>    OSCCLK,
          RST_IN          =>    RST,
          CLKFX_OUT       =>   RCLK,
          CLKIN_IBUFG_OUT =>   CLKIN_IBUFG_OUT1,
          CLK0_OUT        =>   CLK0_OUT1,
          LOCKED_OUT      =>   LOCKED_OUT1);

  -- Phase shifting the global clock with 180 degrees:

	global_phase_shift0: global_phase_shift PORT MAP(
		CLKIN_IN => CLK_GLOBAL,
		RST_IN => RST,
		CLKIN_IBUFG_OUT => CLKIN_IBUFG_OUT2,
		CLK0_OUT => CLK0_OUT2,
		CLK180_OUT => CLK_GLOBAL_180,
		LOCKED_OUT => LOCKED_OUT2
	);

  -- Do not modify any port, signal, or instance
  -- names related to the PCI Interface!

  PCI_CORE : pcim_lc port map (
    AD_IO               => AD,
    CBE_IO              => CBE,
    PAR_IO              => PAR,
    PAR64_IO            => PAR64,
    FRAME_IO            => FRAME_N,
    REQ64_IO            => REQ64_N,
    TRDY_IO             => TRDY_N,
    IRDY_IO             => IRDY_N,
    STOP_IO             => STOP_N,
    DEVSEL_IO           => DEVSEL_N,
    ACK64_IO            => ACK64_N,
    IDSEL_I             => IDSEL,
    INTA_O              => INTR_A,
    PERR_IO             => PERR_N,
    SERR_IO             => SERR_N,
    REQ_O               => REQ_N,
    GNT_I               => GNT_N,
    RST_I               => RST_N,
    PCLK                => PCLK,
    RCLK                => RCLK,

    FRAMEQ_N            => FRAMEQ_N,
    REQ64Q_N            => REQ64Q_N,
    TRDYQ_N             => TRDYQ_N,
    IRDYQ_N             => IRDYQ_N,
    STOPQ_N             => STOPQ_N,
    DEVSELQ_N           => DEVSELQ_N,
    ACK64Q_N            => ACK64Q_N,
    ADDR                => ADDR,
    ADIO                => ADIO,
    CFG_VLD             => CFG_VLD,
    CFG_HIT             => CFG_HIT,
    C_TERM              => C_TERM,
    C_READY             => C_READY,
    ADDR_VLD            => ADDR_VLD,
    BASE_HIT            => BASE_HIT,
    S_CYCLE64           => S_CYCLE64,
    S_TERM              => S_TERM,
    S_READY             => S_READY,
    S_ABORT             => S_ABORT,
    S_WRDN              => S_WRDN,
    S_SRC_EN            => S_SRC_EN,
    S_DATA_VLD          => S_DATA_VLD,
    S_CBE               => S_CBE,
    PCI_CMD             => PCI_CMD,
    REQUEST             => REQUEST,
    REQUEST64           => REQUEST64,
    REQUESTHOLD         => REQUESTHOLD,
    COMPLETE            => COMPLETE,
    M_WRDN              => M_WRDN,
    M_READY             => M_READY,
    M_SRC_EN            => M_SRC_EN,
    M_DATA_VLD          => M_DATA_VLD,
    M_CBE               => M_CBE,
    TIME_OUT            => TIME_OUT,
    M_FAIL64            => M_FAIL64,
    CFG_SELF            => CFG_SELF,
    M_DATA              => M_DATA,
    DR_BUS              => DR_BUS,
    I_IDLE              => I_IDLE,
    M_ADDR_N            => M_ADDR_N,
    IDLE                => IDLE,
    B_BUSY              => B_BUSY,
    S_DATA              => S_DATA,
    BACKOFF             => BACKOFF,
    SLOT64              => SLOT64,
    INTR_N              => INTR_N,
    PERRQ_N             => PERRQ_N,
    SERRQ_N             => SERRQ_N,
    KEEPOUT             => KEEPOUT,
    CSR                 => CSR,
    SUB_DATA            => SUB_DATA,
    CFG                 => CFG_BUS,
    RST                 => RST,
    CLK                 => CLK
  );


  -- Instantiate the configuration module

  CFG_INST : CFG port map (CFG => CFG_BUS);


  -- Instantiate the user application

  BACKEND : local_side port map (
    FRAMEQ_N            => FRAMEQ_N,
    REQ64Q_N            => REQ64Q_N,
    TRDYQ_N             => TRDYQ_N,
    IRDYQ_N             => IRDYQ_N,
    STOPQ_N             => STOPQ_N,
    DEVSELQ_N           => DEVSELQ_N,
    ACK64Q_N            => ACK64Q_N,
    ADDR                => ADDR,
    ADIO                => ADIO,
    CFG_VLD             => CFG_VLD,
    CFG_HIT             => CFG_HIT,
    C_TERM              => C_TERM,
    C_READY             => C_READY,
    ADDR_VLD            => ADDR_VLD,
    BASE_HIT            => BASE_HIT,
    S_CYCLE64           => S_CYCLE64,
    S_TERM              => S_TERM,
    S_READY             => S_READY,
    S_ABORT             => S_ABORT,
    S_WRDN              => S_WRDN,
    S_SRC_EN            => S_SRC_EN,
    S_DATA_VLD          => S_DATA_VLD,
    S_CBE               => S_CBE,
    PCI_CMD             => PCI_CMD,
    REQUEST             => REQUEST,
    REQUEST64           => REQUEST64,
    REQUESTHOLD         => REQUESTHOLD,
    COMPLETE            => COMPLETE,
    M_WRDN              => M_WRDN,
    M_READY             => M_READY,
    M_SRC_EN            => M_SRC_EN,
    M_DATA_VLD          => M_DATA_VLD,
    M_CBE               => M_CBE,
    TIME_OUT            => TIME_OUT,
    M_FAIL64            => M_FAIL64,
    CFG_SELF            => CFG_SELF,
    M_DATA              => M_DATA,
    DR_BUS              => DR_BUS,
    I_IDLE              => I_IDLE,
    M_ADDR_N            => M_ADDR_N,
    IDLE                => IDLE,
    B_BUSY              => B_BUSY,
    S_DATA              => S_DATA,
    BACKOFF             => BACKOFF,
    SLOT64              => SLOT64,
    INTR_N              => INTR_N,
    PERRQ_N             => PERRQ_N,
    SERRQ_N             => SERRQ_N,
    KEEPOUT             => KEEPOUT,
    CSR                 => CSR,
    SUB_DATA            => SUB_DATA,
    CFG                 => CFG_BUS,
    RST                 => RST,
    CLK                 => CLK_GLOBAL_180,
    CLK_REG			=> CLK, -- Innputt for regional klokke
    roCLK        			=> roCLK,
    roRST_N      			=> roRST_N,
    riLD_N       			=> riLD_N,
    riLF_N       			=> riLF_N,
    roBSY_N      			=> roBSY_N,
    roD          			=> roD,
    roTEN_N      			=> roTEN_N,
    roCMD_N      			=> roCMD_N,
    riD          			=> riD,
    riTEN_N      			=> riTEN_N,
    riSTS_N      			=> riSTS_N,
    sram32_a     			=> sram32_a_int,
    sram32_d     			=> ucc_d,
    sram32_we    			=> ucc_we,
    sram32_be    			=> ucc_be,
    sram32_oe    			=> sram32_oe,
    sram64_a     			=> mem_a_int,
    sram64_d     			=> mem_d,
    sram64_we    			=> sram64_we,
    sram64_be    			=> sram64_be,
    sram64_oe    			=> sram64_oe,
    sram64_cs    			=> sram64_cs,
    hostbuf_full 			=> hostbuf_full,
    dmabuf_full  			=> dmabuf_full,
    dma_enabled  			=> dma_enabled,
    dma_active   			=> dma_active,
    link_down    			=> link_down
  );
  
 
end rtl;
