----------------------------------------------------------------------------
--
--  File:   ping.vhd
--  Rev:    3.0.0
--
--  This is an example user application for use with the Xilinx PCI
--  initiator.  The design has been tested with the Synopsys design
--  flow.
--
--  Copyright (c) 2005 Xilinx, Inc.  All rights reserved.
--
----------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
-- synopsys translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- synopsys translate_on


entity ping64 is
  port (-- Interface to PCI Logicore.
        FRAMEQ_N        : in    std_logic;
        REQ64Q_N        : in    std_logic;
        TRDYQ_N         : in    std_logic;
        IRDYQ_N         : in    std_logic;
        STOPQ_N         : in    std_logic;
        DEVSELQ_N       : in    std_logic;
        ACK64Q_N        : in    std_logic;

        ADDR            : in    std_logic_vector( 31 downto 0);
        ADIO            : inout std_logic_vector( 63 downto 0);

        CFG_VLD         : in    std_logic;
        CFG_HIT         : in    std_logic;
        C_TERM          : out   std_logic;
        C_READY         : out   std_logic;

        ADDR_VLD        : in    std_logic;
        BASE_HIT        : in    std_logic_vector(  7 downto 0);

        S_CYCLE64       : in    std_logic;
        S_TERM          : out   std_logic;
        S_READY         : out   std_logic;
        S_ABORT         : out   std_logic;
        S_WRDN          : in    std_logic;
        S_SRC_EN        : in    std_logic;
        S_DATA_VLD      : in    std_logic;
        S_CBE           : in    std_logic_vector(  7 downto 0);
        PCI_CMD         : in    std_logic_vector( 15 downto 0);

        REQUEST         : out   std_logic;
        REQUEST64       : out   std_logic;
        REQUESTHOLD     : out   std_logic;
        COMPLETE        : out   std_logic;

        M_WRDN          : out   std_logic;
        M_READY         : out   std_logic;
        M_SRC_EN        : in    std_logic;
        M_DATA_VLD      : in    std_logic;
        M_CBE           : out   std_logic_vector(  7 downto 0);

        TIME_OUT        : in    std_logic;
        M_FAIL64        : in    std_logic;
        CFG_SELF        : out   std_logic;

        M_DATA          : in    std_logic;
        DR_BUS          : in    std_logic;
        I_IDLE          : in    std_logic;
        M_ADDR_N        : in    std_logic;
        IDLE            : in    std_logic;
        B_BUSY          : in    std_logic;
        S_DATA          : in    std_logic;
        BACKOFF         : in    std_logic;

        SLOT64          : out   std_logic;
        INTR_N          : out   std_logic;
        PERRQ_N         : in    std_logic;
        SERRQ_N         : in    std_logic;
        KEEPOUT         : out   std_logic;

        CSR             : in    std_logic_vector( 39 downto 0);
        SUB_DATA        : out   std_logic_vector( 31 downto 0);
        CFG             : in    std_logic_vector(255 downto 0);

        RST             : in    std_logic;
        CLK             : in    std_logic;

        -- Add user I/O ports here

        PING_DONE       : out   std_logic;
        PING_REQUEST32  : in    std_logic;
        PING_REQUEST64  : in    std_logic
  );
end ping64;


architecture rtl of ping64 is

  attribute syn_edif_bit_format : string;
  attribute syn_edif_scalar_format : string;
  attribute syn_noclockbuf : boolean;
  attribute syn_hier : string;
  attribute syn_edif_bit_format of rtl : architecture is "%u<%i>";
  attribute syn_edif_scalar_format of rtl : architecture is "%u";
  attribute syn_noclockbuf of rtl : architecture is true;
  attribute syn_hier of rtl : architecture is "hard";
  
  component IBUF_PCI33_3 
    port(O: out STD_LOGIC; 
         I: in STD_LOGIC 
    ); 
  end component; 
  
  component OBUF_PCI33_3 
    port(O: out STD_LOGIC;
         I: in STD_LOGIC
    );   
  end component; 
 
  type   fsm_states is  (IDLE_S, WRITE32_S, READ32_S, WRITE64_S, READ64_S);
  signal ping_state     : fsm_states;
  signal nxt_ping_state : fsm_states;

  signal cfg_rd         : std_logic;
  signal bar0_rd        : std_logic;
  signal bar1_rd        : std_logic;
  signal bar2_rd        : std_logic;

  signal cfg_wr         : std_logic;
  signal bar0_wr        : std_logic;
  signal bar1_wr        : std_logic;
  signal bar2_wr        : std_logic;

  signal cfg_rd_cs      : std_logic;
  signal bar0_rd_cs     : std_logic;
  signal bar1_rd_cs     : std_logic;
  signal bar2_rd_cs     : std_logic;

  signal cfg_wr_cs      : std_logic;
  signal bar0_wr_cs     : std_logic;
  signal bar1_wr_cs     : std_logic;
  signal bar2_wr_cs     : std_logic;

  signal my_cfg_reg     : std_logic_vector(31 downto 0);
  signal oe_cfg_reg     : std_logic;
  signal en_cfg_reg     : std_logic;

  signal my_io_reg      : std_logic_vector(31 downto 0);
  signal oe_io_reg      : std_logic;

  signal my_mem_reg     : std_logic_vector(31 downto 0);
  signal oe_mem_reg     : std_logic;

  signal my_wide_reg    : std_logic_vector(63 downto 0);
  signal oe_wide_reg    : std_logic;

  signal xfer_len       : std_logic_vector( 3 downto 0);
  signal xfer_load_delay: std_logic;
  signal mdata_delay    : std_logic;
  signal feedback       : std_logic;
  signal pre_done       : std_logic;
  signal reg_preq32     : std_logic;
  signal reg_preq64     : std_logic;
  signal ns_done        : std_logic;
  signal mdata_fell     : std_logic;
  signal xfer_load      : std_logic;
  signal start32        : std_logic;
  signal start64        : std_logic;
  signal dir            : std_logic;
  signal cnt3           : std_logic;
  signal cnt2           : std_logic;
  signal cnt1           : std_logic;
  signal fin3           : std_logic;
  signal fin2           : std_logic;
  signal fin1           : std_logic;
  signal assert_complete: std_logic;
  signal hold_complete  : std_logic;

  signal my_init_reg    : std_logic_vector(63 downto 0);
  signal oe_init_reg    : std_logic;
  signal xlr, xlw, xhr  : std_logic;

  signal s_term_reg     : std_logic;
  signal s_ready_reg    : std_logic;
  signal m_ready_reg    : std_logic;
  signal intr_n_reg     : std_logic;

  signal ping_done_o    : std_logic; 
  signal ping_req32_i   : std_logic;
  signal ping_req64_i   : std_logic;

begin

  -----------------------------------------------------------------------
  -- This section contains the PCI interface decode.
  -----------------------------------------------------------------------

  IDENTIFY:
  process(RST, CLK)
  begin
    if RST = '1' then
      cfg_rd <= '0';
      cfg_wr <= '0';
      bar0_rd <= '0';
      bar0_wr <= '0';
      bar1_rd <= '0';
      bar1_wr <= '0';
      bar2_rd <= '0';
      bar2_wr <= '0';
    elsif (CLK'event and CLK = '1') then
      if CFG_HIT = '1' then
        cfg_rd <= not S_WRDN;
        cfg_wr <= S_WRDN;
      elsif S_DATA = '0' then
        cfg_rd <= '0';
        cfg_wr <= '0';
      end if;
      if BASE_HIT(0) = '1' then
        bar0_rd <= not S_WRDN;
        bar0_wr <= S_WRDN;
      elsif S_DATA = '0' then
        bar0_rd <= '0';
        bar0_wr <= '0';
      end if;
      if BASE_HIT(1) = '1' then
        bar1_rd <= not S_WRDN;
        bar1_wr <= S_WRDN;
      elsif S_DATA = '0' then
        bar1_rd <= '0';
        bar1_wr <= '0';
      end if;
      if BASE_HIT(2) = '1' then
        bar2_rd <= not S_WRDN;
        bar2_wr <= S_WRDN;
      elsif S_DATA = '0' then
        bar2_rd <= '0';
        bar2_wr <= '0';
      end if;
    end if;
  end process;
 
  cfg_rd_cs  <= cfg_rd;
  cfg_wr_cs  <= cfg_wr;
  bar0_rd_cs <= bar0_rd;
  bar0_wr_cs <= bar0_wr;
  bar1_rd_cs <= bar1_rd;
  bar1_wr_cs <= bar1_wr;
  bar2_rd_cs <= bar2_rd;
  bar2_wr_cs <= bar2_wr;


  -----------------------------------------------------------------------
  -- This section contains the CFG32 implementation.
  -----------------------------------------------------------------------

  WRITE_MY_CFG_REG:
  process(RST, CLK)
  begin
    if RST = '1' then my_cfg_reg <= "00000000000000000000000000000000";
    elsif (CLK'event and CLK = '1') then
      if S_DATA_VLD = '1' and cfg_wr_cs = '1' and en_cfg_reg = '1' then

        if S_CBE(0) = '0' then my_cfg_reg( 7 downto  0) <= ADIO( 7 downto  0);
        end if;
        if S_CBE(1) = '0' then my_cfg_reg(15 downto  8) <= ADIO(15 downto  8);
        end if;
        if S_CBE(2) = '0' then my_cfg_reg(23 downto 16) <= ADIO(23 downto 16);
        end if;
        if S_CBE(3) = '0' then my_cfg_reg(31 downto 24) <= ADIO(31 downto 24);
        end if;

      end if;
    end if;
  end process;

  en_cfg_reg <= ADDR(7) or ADDR(6);
  oe_cfg_reg <= en_cfg_reg and cfg_rd_cs and S_DATA and CFG(118);
  ADIO(31 downto 0) <= my_cfg_reg when (oe_cfg_reg = '1') else (others=>'Z');


  -----------------------------------------------------------------------
  -- This section contains the IO32 implementation.
  -----------------------------------------------------------------------
 
  WRITE_MY_IO_REG:
  process(RST, CLK)
  begin
    if RST = '1' then my_io_reg <= "00010000000100000001000000010000";
    elsif (CLK'event and CLK = '1') then
      if S_DATA_VLD = '1' and bar0_wr_cs = '1' then
 
        if S_CBE(0) = '0' then my_io_reg( 7 downto  0) <= ADIO( 7 downto  0);
        end if;
        if S_CBE(1) = '0' then my_io_reg(15 downto  8) <= ADIO(15 downto  8);
        end if;
        if S_CBE(2) = '0' then my_io_reg(23 downto 16) <= ADIO(23 downto 16);
        end if;
        if S_CBE(3) = '0' then my_io_reg(31 downto 24) <= ADIO(31 downto 24);
        end if;
 
      end if;
    end if;
  end process;
 
  oe_io_reg <= bar0_rd_cs and S_DATA;
  ADIO(31 downto 0) <= my_io_reg when (oe_io_reg = '1') else (others=>'Z');


  -----------------------------------------------------------------------
  -- This section contains the MEM32 implementation.  
  -----------------------------------------------------------------------
 
  WRITE_MY_MEM_REG:
  process(RST, CLK)
  begin  
    if RST = '1' then my_mem_reg <= "00110010001100100011001000110010";
    elsif (CLK'event and CLK = '1') then
      if S_DATA_VLD = '1' and bar1_wr_cs = '1' then
 
        if S_CBE(0) = '0' then my_mem_reg( 7 downto  0) <= ADIO( 7 downto  0);
        end if;  
        if S_CBE(1) = '0' then my_mem_reg(15 downto  8) <= ADIO(15 downto  8);
        end if;  
        if S_CBE(2) = '0' then my_mem_reg(23 downto 16) <= ADIO(23 downto 16);
        end if;  
        if S_CBE(3) = '0' then my_mem_reg(31 downto 24) <= ADIO(31 downto 24);
        end if; 
 
      end if;
    end if; 
  end process;

  oe_mem_reg <= bar1_rd_cs and S_DATA;
  ADIO(31 downto 0) <= my_mem_reg when (oe_mem_reg = '1') else (others=>'Z');


  ----------------------------------------------------------------------- 
  -- This section contains the MEM64 implementation.   
  -----------------------------------------------------------------------
  
  WRITE_MY_WIDE_REG: 
  process(RST, CLK) 
  begin   
    if RST = '1' then my_wide_reg <= "01100100011001000110010001100100" &
                                     "01100100011001000110010001100100";
    elsif (CLK'event and CLK = '1') then 
      if S_DATA_VLD = '1' and bar2_wr_cs = '1' then 
  
        if S_CBE(0) = '0' then my_wide_reg( 7 downto  0) <= ADIO( 7 downto  0); 
        end if;   
        if S_CBE(1) = '0' then my_wide_reg(15 downto  8) <= ADIO(15 downto  8); 
        end if;   
        if S_CBE(2) = '0' then my_wide_reg(23 downto 16) <= ADIO(23 downto 16); 
        end if;   
        if S_CBE(3) = '0' then my_wide_reg(31 downto 24) <= ADIO(31 downto 24); 
        end if;  
  
        if S_CBE(4) = '0' then my_wide_reg(39 downto 32) <= ADIO(39 downto 32); 
        end if;   
        if S_CBE(5) = '0' then my_wide_reg(47 downto 40) <= ADIO(47 downto 40); 
        end if;   
        if S_CBE(6) = '0' then my_wide_reg(55 downto 48) <= ADIO(55 downto 48); 
        end if;   
        if S_CBE(7) = '0' then my_wide_reg(63 downto 56) <= ADIO(63 downto 56); 
        end if;  

      end if; 
    end if;  
  end process; 
  
  oe_wide_reg <= bar2_rd_cs and S_DATA; 
  ADIO(63 downto 0) <= my_wide_reg when (oe_wide_reg = '1') else (others=>'Z');


  ----------------------------------------------------------------------- 
  -- This section contains the initiator logic.
  ----------------------------------------------------------------------- 

  -- Bus addresses are obtained from the MEM32 register.
  -- Direction is from IO32(31) and burst length is IO32(3:0).
  -- A general purpose register file is used for the source
  -- or destination depending on the data transfer direction.

  PING_FSM:
  process(ping_state, start32, start64, mdata_fell, dir)
  begin
    case ping_state is

      -- IDLE_S is the idle state.  If the state machine is
      -- signaled to start, proceed to the next state.
      when IDLE_S =>
        if start64 = '1' and dir = '1' then nxt_ping_state <= WRITE64_S;
        elsif start64 = '1' and dir = '0' then nxt_ping_state <= READ64_S;
        elsif start32 = '1' and dir = '1' then nxt_ping_state <= WRITE32_S;
        elsif start32 = '1' and dir = '0' then nxt_ping_state <= READ32_S;
        else nxt_ping_state <= IDLE_S;
        end if;

      -- WRITE64_S stays put until it sees a deassertion of
      -- the M_DATA signal indicating that a transfer is over.
      -- More elaborate FSMs may check error conditions at
      -- the time mdata_fell is asserted.
      when WRITE64_S =>
        if mdata_fell = '1' then nxt_ping_state <= IDLE_S;
        else nxt_ping_state <= WRITE64_S;
        end if;

      -- WRITE32_S stays put until it sees a deassertion of
      -- the M_DATA signal indicating that a transfer is over.
      -- More elaborate FSMs may check error conditions at
      -- the time mdata_fell is asserted.
      when WRITE32_S =>
        if mdata_fell = '1' then nxt_ping_state <= IDLE_S;
        else nxt_ping_state <= WRITE32_S;
        end if;

      -- READ64_S stays put until it sees a deassertion of
      -- the M_DATA signal indicating that a transfer is over.
      -- More elaborate FSMs may check error conditions at
      -- the time mdata_fell is asserted.
      when READ64_S =>
        if mdata_fell = '1' then nxt_ping_state <= IDLE_S;
        else nxt_ping_state <= READ64_S;
        end if;

      -- READ32_S stays put until it sees a deassertion of
      -- the M_DATA signal indicating that a transfer is over.
      -- More elaborate FSMs may check error conditions at
      -- the time mdata_fell is asserted.
      when READ32_S =>
        if mdata_fell = '1' then nxt_ping_state <= IDLE_S;
        else nxt_ping_state <= READ32_S;
        end if;
 
      -- Include a default state just in case we have any
      -- accidents with the state machine.
      when others =>
        nxt_ping_state <= IDLE_S;
    end case;
  end process;
 
  PING_FSM_SEQ:
  process(RST, CLK)
  begin
    if RST = '1' then ping_state <= IDLE_S;
    elsif (CLK'event and CLK = '1') then ping_state <= nxt_ping_state;
    end if;
  end process;

  -- Need a delayed version of M_DATA and also
  -- a delayed version of the transfer length
  -- counter load signal.

  MISC_SIGNALS:
  process(RST, CLK)
  begin
    if RST = '1' then
      mdata_delay <= '0';
      xfer_load_delay <= '0';
      pre_done <= '0';
      reg_preq32 <= '0';
      reg_preq64 <= '0';
    elsif (CLK'event and CLK = '1') then
      mdata_delay <= M_DATA;
      xfer_load_delay <= xfer_load;
      pre_done <= ns_done;
      reg_preq32 <= ping_req32_i;
      reg_preq64 <= ping_req64_i;
    end if; 
  end process;

--  Fjernar tilkoplingane til paddane for aa laga dummy-design: OT

--  XPCI_PING_REQ32 : IBUF_PCI33_3 port map
--    ( O => ping_req32_i, I => PING_REQUEST32 );
--  XPCI_PING_REQ64 : IBUF_PCI33_3 port map
--    ( O => ping_req64_i, I => PING_REQUEST64 );

  -- This is the "set/reset" implementation
  -- for the COMPLETE logic.
  
  ping_req64_i <= '0';
  ping_req32_i <= '0';

  HOLD_IT:
  process(RST, CLK)
  begin
    if RST = '1' then feedback <= '0';
    elsif (CLK'event and CLK = '1') then
      if mdata_fell = '1' then feedback <= '0';
      elsif assert_complete = '1' then feedback <= '1';
      end if;
    end if;
  end process;

  -- This is the transfer length counter.
  -- Transfer lengths may be anywhere from
  -- one to sixteen data phases.

  TRANSFER_COUNTER:
  process(RST, CLK)
  begin
    if RST = '1' then xfer_len <= "0000";
    elsif (CLK'event and CLK = '1') then
      if xfer_load = '1' then xfer_len <= my_io_reg(3 downto 0);
      elsif M_DATA_VLD = '1' then xfer_len <= xfer_len - 1;
      end if;
    end if;
  end process;

  -- Decode some things for the complete logic.

  cnt3 <= '1' when (xfer_len = "0011") else '0';
  cnt2 <= '1' when (xfer_len = "0010") else '0';
  cnt1 <= '1' when (xfer_len = "0001") else '0';
  fin3 <= cnt3 and M_DATA_VLD;
  fin2 <= cnt2 and mdata_delay;
  fin1 <= cnt1 and xfer_load_delay;

  -- Generate some internal signals.

  start32 <= reg_preq32;
  start64 <= reg_preq64;
  dir <= my_io_reg(31);
  mdata_fell <= not M_DATA and mdata_delay;
  xfer_load <= (start32 or start64) when (ping_state = IDLE_S) else '0';
  assert_complete <= fin1 or fin2 or fin3;
  hold_complete <= feedback;
  ns_done <= mdata_fell when (ping_state /= IDLE_S) else '0';

  -- Drive outputs to the PCI interface.

  M_WRDN <= dir;
  REQUEST <= start32 when (ping_state = IDLE_S) else '0';
  REQUEST64 <= start64 when (ping_state = IDLE_S) else '0';
  COMPLETE <= assert_complete or hold_complete;

--  ping_done_o <= pre_done; -- OT test 15. juni
    ping_done_o <= '1';

  XPCI_PING_DONE : OBUF_PCI33_3 port map ( O => PING_DONE, I => ping_done_o );
  ADIO(63 downto 0) <= ("00000000000000000000000000000000" & my_mem_reg)
                       when (M_ADDR_N = '0') else (others =>'Z');
  M_CBE(7 downto 0) <= ("0000" & my_io_reg(7 downto 5) & dir)
                       when (M_ADDR_N = '0') else (others =>'0');

  -- A simple 64-bit register for data transfer.

  xlr <= '1' when ((ping_state=READ32_S) or (ping_state=READ64_S)) else '0';
  xlw <= '1' when ((ping_state=WRITE32_S) or (ping_state=WRITE64_S)) else '0';
  xhr <= '1' when (ping_state = READ64_S) else '0';
  
  WRITE_MY_INIT_REG: 
  process(RST, CLK) 
  begin   
    if RST = '1' then my_init_reg <= "00000000000000000000000000000000" &
                                     "00000000000000000000000000000000";
    elsif (CLK'event and CLK = '1') then 
      if M_DATA_VLD = '1' then 
  
        if xlr = '1' then my_init_reg( 7 downto  0) <= ADIO( 7 downto  0); 
        end if;   
        if xlr = '1' then my_init_reg(15 downto  8) <= ADIO(15 downto  8); 
        end if;   
        if xlr = '1' then my_init_reg(23 downto 16) <= ADIO(23 downto 16); 
        end if;   
        if xlr = '1' then my_init_reg(31 downto 24) <= ADIO(31 downto 24); 
        end if;  
  
        if xhr = '1' then my_init_reg(39 downto 32) <= ADIO(39 downto 32); 
        end if;   
        if xhr = '1' then my_init_reg(47 downto 40) <= ADIO(47 downto 40); 
        end if;   
        if xhr = '1' then my_init_reg(55 downto 48) <= ADIO(55 downto 48); 
        end if;   
        if xhr = '1' then my_init_reg(63 downto 56) <= ADIO(63 downto 56); 
        end if;  

      end if; 
    end if;  
  end process; 
  
  oe_init_reg <= M_DATA and xlw;
  ADIO(63 downto 0) <= my_init_reg when (oe_init_reg = '1') else (others=>'Z');


  ----------------------------------------------------------------------- 
  -- This section contains unused signals.
  ----------------------------------------------------------------------- 

  C_TERM <= '1';
  C_READY <= '1';
  KEEPOUT <= '0';
  CFG_SELF <= '0';
  REQUESTHOLD <= '0';
  SUB_DATA <= "00000000000000000000000000000000";

  DONT_OPTIMIZE_PLEASE:
  process(RST, CLK)
  begin
    if RST = '1' then
      intr_n_reg <= '0';
      s_term_reg <= '1';
      s_ready_reg <= '0';
      m_ready_reg <= '0';
    elsif (CLK'event and CLK = '1') then
      intr_n_reg <= '1';
      s_term_reg <= '0';
      s_ready_reg <= '1';
      m_ready_reg <= '1';
    end if;
  end process;
 
  INTR_N <= intr_n_reg;
  S_TERM <= s_term_reg;
  S_READY <= s_ready_reg;
  S_ABORT <= S_CYCLE64 and ADDR(2);
  M_READY <= m_ready_reg;
  SLOT64 <= not my_cfg_reg(0);


end rtl;
