/*******************************************************************************

  Author: Olav Torheim
  Email:  torheim@ift.uib.no

  This sample code uses the rorcdriver module to communicate with the HLT-RORC card.
  Parts of this code come from other applications created by Artur Szostak (artur@alice.phy.uct.ac.za).

********************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <unistd.h>
#include <time.h>

#include <unistd.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <signal.h>
#include "rorcdriver.h"

void cleanup(int signal) // Cleanup catches the SIGINT signal when the program is terminated with Ctrl-C
{
	rorcdriver_status_t status;

	// Unregisters from the rorcdriver.o driver.
	printf("No skal me avregistrera oss.\n");
	status = rorcdriverUnregister();
	printf("No hev me avregistrert oss.\n");
	if (status != RORCDRIVER_OK)
	{
		printf("ERROR: %s\n", rorcdriverStatusString(status) );
	};

	printf("Done.\n");
	exit(0);
};



int main (int argc, char **argv)
{

	int result;
	int buffer_length = 256;
	rorcdriver_status_t status;

	// Register with the rorcdriver.o driver.
	status = rorcdriverRegister(buffer_length, buffer_length,0x8, 0x1B, 1000, 1, 1, 1); // 1 sekund til timeout = 1000 millisekund

	if (status != RORCDRIVER_OK)
	{
		printf("ERROR: %s\n", rorcdriverStatusString(status) );
		return 1;
	};


	// Must end the program with ctrl-c, since we go into infinite loop (but for the moment this loop is commented out, so never mind)
	struct sigaction close_program;
	close_program.sa_handler = cleanup;
	sigemptyset(&close_program.sa_mask);
	close_program.sa_flags = 0;
	result = sigaction(SIGINT, &close_program, 0);
	if (result != 0)
	{
		perror("ERROR");
		return 1;
	};


        int *addr;
        addr = (int *)rorcdriverOpenEventBuffer((0x400 << 3));
      	printf("Peikar til adresse %8.8lX\n", addr);

        int rapporteringsord;
        int teljar;
        int teljar2;

        for(teljar2=0;teljar2<buffer_length;teljar2=teljar2+64)
        {
          rapporteringsord = rorcdriverGetReportWord();
          if (rapporteringsord == -1)
          {
            printf("Request TIMEOUT!\n");
          }
          else
          {
                    printf("Rapporteringsordet er %X\nDesimalt: %d\n", rapporteringsord, rapporteringsord);
                    for(teljar = 0; teljar < 64; teljar++)
                    {
                     addr[buffer_length+teljar+teljar2] = addr[teljar+teljar2] -1;
                     printf("Les ut %X\n", addr[teljar+teljar2]);
                    }
                    rorcdriverProcessingCompleted();
          }
        }

              cleanup(0);
/*
	do
	{
		sleep(10);
	} while (1);
*/

	return 0;




};
