/*********************************************************************************

   Author: Olav Torheim (torheim@ift.uib.no)

   Declarations for the rorcdriver API library.
   Parts of this code come from the irqsignal API created by Artur Szostak (artur@alice.phy.uct.ac.za).

**********************************************************************************/

#ifndef IRQ_SIGNAL_API_H
#define IRQ_SIGNAL_API_H

#include "../src/rorcdriver_mod.h"

#ifdef __cplusplus
extern "C"
{
#endif


/* The registration routine.

   Returns:
     RORCDRIVER_OK                 - Registered successfully.
     RORCDRIVER_OUTOFMEM           - Could not allocate required system memory.
     RORCDRIVER_UNKNOWN_ERROR      - The kernel routine "request_irq" returned a unexpected result.
     RORCDRIVER_CANT_OPEN_DRIVER   - Could not open the driver. Perhaps the node /dev/rorcdriver is missing.
 */
extern rorcdriver_status_t rorcdriverRegister(unsigned long event_buffer_length, unsigned long report_buffer_length, unsigned long irq_pulse_length, unsigned long event_length, unsigned long timeout_in_millisecs, int enable_dma_transfers, int enable_irq_transfers, int enable_pg_transfers);

/* The unregistration routine.

   Returns:
     RORCDRIVER_OK               - Unregistered successfully.
     RORCDRIVER_CANT_OPEN_DRIVER - Could not open the driver. Perhaps the node /dev/rorcdriver is missing.
 */
extern rorcdriver_status_t rorcdriverUnregister(void);

/* Retuning a pointer to the memory mapped DMA buffers */
extern int * rorcdriverOpenEventBuffer(int length);

/* Returning event length in words */
extern int rorcdriverGetReportWord(void);

/* Reporting back to the driver that the application has finished processing a whole event */
extern void rorcdriverProcessingCompleted(void);

#ifdef DEBUG

/* Removes all registration information. This function should only be used for debugging.

   Returns:
     RORCDRIVER_OK               - Successful reset.
     RORCDRIVER_CANT_OPEN_DRIVER - Could not open the driver. Perhaps the node /dev/rorcdriver is missing.
 */
extern rorcdriver_status_t rorcdriverReset(unsigned int irq);

/* Removes all registration information. This function should only be used for debugging.

   Returns:
     RORCDRIVER_OK               - Successful reset.
     RORCDRIVER_CANT_OPEN_DRIVER - Could not open the driver. Perhaps the node /dev/rorcdriver is missing.
 */
extern rorcdriver_status_t rorcdriverResetAll(void);

/* Clears all internal structures and resets the module usage count to 1.

   Returns:
     RORCDRIVER_OK               - Successful reset.
     RORCDRIVER_CANT_OPEN_DRIVER - Could not open the driver. Perhaps the node /dev/rorcdriver is missing.
 */
extern rorcdriver_status_t rorcdriverHardReset(void);

/* Used for testing and debugging. This routine simulates a request from the application to the
   rorcdriver kernel module.

   Returns:
     RORCDRIVER_OK                 - Routine exited successfully. Does not indicate
                                     a succesfull test though.
     RORCDRIVER_CANT_SCHEDULE_TASK - Something went wrong when trying to schedule the
                                     dispatcher task.
     RORCDRIVER_CANT_OPEN_DRIVER   - Could not open the driver. Perhaps the node /dev/rorcdriver is missing.
 */

#endif // DEBUG


/* Returns a status message corresponding to the given status code. If the code is unknown
   then we return the result of strerror(status).
 */
extern char* rorcdriverStatusString(rorcdriver_status_t status);


#ifdef __cplusplus
};
#endif

#endif // IRQ_SIGNAL_API_H













