/*********************************************************************************

   Author: Olav Torheim
   Email:  torheim@ift.uib.no

   Declarations for the rorcdriver kernel module.
   Parts of this code come from the irqsignal module created by Artur Szostak (artur@alice.phy.uct.ac.za).

**********************************************************************************/

#ifndef RORCDRIVER_KERNEL_MODULE_H
#define RORCDRIVER_KERNEL_MODULE_H

// The possible error codes returned by this module on any of the ioctl calls.                 
typedef enum rorcdriver_status
{
	RORCDRIVER_OK = 0,
	RORCDRIVER_UNKNOWN_ERROR = -1,
	RORCDRIVER_NO_SUCH_IOCTRL = 100,
	RORCDRIVER_OUTOFMEM = 101,
	RORCDRIVER_CANT_SCHEDULE_TASK = 400,

	RORCDRIVER_CANT_OPEN_DRIVER = 20  // This error code is actually returned by the API.
} rorcdriver_status_t;

typedef struct rorcdriver_register_info
{
        unsigned long event_buffer_length;
        unsigned long report_buffer_length;
        unsigned long irq_pulse_length;
        unsigned long event_length;
        unsigned long timeout_in_millisecs;
        int enable_dma_transfers;
        int enable_irq_transfers;
        int enable_pg_transfers;
} rorcdriver_register_info_t;


#include <asm/ioctl.h>

// The base IOCTL number.
#define rorcdriver_TYPE_NUMBER 'I'

// All the actuall IOCTL numbers used by the kernel module.
#define rorcdriver_REGISTER			_IOC( _IOC_READ, rorcdriver_TYPE_NUMBER, 0, sizeof(rorcdriver_register_info_t) )
#define rorcdriver_UNREGISTER		_IOC( _IOC_READ, rorcdriver_TYPE_NUMBER, 1, sizeof(rorcdriver_register_info_t) )
#define rorcdriver_RESET			_IOC( _IOC_NONE, rorcdriver_TYPE_NUMBER, 2, sizeof(unsigned int) )
#define rorcdriver_RESETALL		_IOC( _IOC_NONE, rorcdriver_TYPE_NUMBER, 3, 0 )
#define rorcdriver_HARDRESET			_IOC( _IOC_NONE, rorcdriver_TYPE_NUMBER, 4, 0 )
#define rorcdriver_GETREPORTWORD				_IOC( _IOC_READ, rorcdriver_TYPE_NUMBER, 5, sizeof(unsigned int) )
#define rorcdriver_PROCESSINGCOMPLETED			_IOC( _IOC_READ, rorcdriver_TYPE_NUMBER, 6, sizeof(unsigned int) )

#endif // RORCDRIVER_KERNEL_MODULE_H
